#include "obstacles.h"

#define OBSTACLE_WIDTH 16
#define OBSTACLE_HEIGHT 130

#define INITIAL_OPENING_HEIGHT 52
#define MINIMUM_OPENING_HEIGHT 40
#define GRAVITY_PER_FRAME 0.06f
#define OBSTACLE_SPEED 2
#define VERTICAL_SHIFT_VELOCITY 1.9f

int p_openingHeight, p_openingMinimumY, p_openingMaximumY;
int p_obstacleX;
float p_floatObstacleOpeningY;
float p_downwardVelocity;

int p_totalCleared;

void p_newObstacle() {
	p_floatObstacleOpeningY = random( p_openingMaximumY - p_openingMinimumY ) + p_openingMinimumY;
	p_obstacleX = SCREEN_WIDTH;
	p_downwardVelocity = 0.0f;
}

void p_setOpeningParams( int openingHeight ) {
	p_openingHeight = openingHeight;
	p_openingMinimumY = PLAYAREA_TOP + 5;
	p_openingMaximumY = PLAYAREA_BOTTOM - p_openingHeight - 5;
}

void obstacles_init() {	
	p_setOpeningParams( INITIAL_OPENING_HEIGHT );
	p_newObstacle();
	p_totalCleared = 0;
}

void p_increaseDifficulty() {
	if( p_openingHeight > MINIMUM_OPENING_HEIGHT ) {
		p_setOpeningParams( p_openingHeight - 1 );
	}
}

void obstacles_render() {
	Rectangle dest;
	
	dest.x = p_obstacleX;
	dest.y = (int)p_floatObstacleOpeningY - OBSTACLE_HEIGHT;
	screen_render( img_obstacle, &dest );
	
	dest.x = p_obstacleX;
	dest.y = (int)p_floatObstacleOpeningY + p_openingHeight;
	screen_render( img_obstacle, &dest );
	
	// draw black "caps" above and below obstacle,
	// so that the obstacles don't "leak" outside of the play area
	dest.x = p_obstacleX;
	dest.y = 0;
	screen_render( img_cap, &dest );
	dest.x = p_obstacleX;
	dest.y = PLAYAREA_BOTTOM;
	screen_render( img_cap, &dest );
}

int p_isCollision( Rectangle* first, Rectangle* second ) {

	if( first->x + first->w <= second->x ||
	    second->x + second->w <= first->x 
		) {
		return 0;
	}
	
	if( first->y + first->h <= second->y ||
	    second->y + second->h <= first->y 
		) {
		return 0;
	}
	
	return 1;
}

int obstacles_isCollisionWith( Rectangle* rectangle ) {
	Rectangle top, bottom;
	
	top.x = p_obstacleX;
	top.y = PLAYAREA_TOP;
	top.w = OBSTACLE_WIDTH;
	top.h = (int)p_floatObstacleOpeningY - top.y;
	
	if( p_isCollision( rectangle, &top ) ) {
		return 1;
	}
	
	bottom.x = p_obstacleX;
	bottom.y = (int)p_floatObstacleOpeningY + p_openingHeight;
	bottom.w = OBSTACLE_WIDTH;
	bottom.h = PLAYAREA_BOTTOM - bottom.y;

	if( p_isCollision( rectangle, &bottom ) ) {
		return 1;
	}
	
	return 0;
}

void obstacles_update() {
	
	// shift it horizontally, towards the player
	p_obstacleX -= OBSTACLE_SPEED;
	if( p_obstacleX < 0 - OBSTACLE_WIDTH ) {
		p_totalCleared++;
		p_increaseDifficulty();
		p_newObstacle();
	}
	
	// v = v0 + a*t
	p_downwardVelocity = p_downwardVelocity + GRAVITY_PER_FRAME;
	
	// shift it vertically
	p_floatObstacleOpeningY += p_downwardVelocity;
	
	if( (int)p_floatObstacleOpeningY > p_openingMaximumY ) {
		// fall through the floor, appearing at the tops
		p_downwardVelocity = 0.0f;
		p_floatObstacleOpeningY = p_openingMinimumY;
	}
	if( (int)p_floatObstacleOpeningY < p_openingMinimumY ) {
		p_floatObstacleOpeningY = p_openingMinimumY;
	}
}

void obstacles_shiftUp() {
	p_downwardVelocity = 0 - VERTICAL_SHIFT_VELOCITY;
}

int obstacles_getTotalCleared() {
	return p_totalCleared;
}
