/*
 *     Saturnade
 *          by Sebastian Mihai, 2015
 *          http://sebastianmihai.com
 *
 */
#include <sgl.h>

#include "images.h"
#include "utilities.h"
#include "obstacles.h"

Rectangle p_ship;
Rectangle p_playArea;

int p_lastScore;

void p_init() {
	p_ship.w = 32;
	p_ship.h = 16;
	p_ship.x = 50;
	p_ship.y = PLAYAREA_MIDDLE - p_ship.h / 2;
	
	p_playArea.x = 0;
	p_playArea.y = BLACK_STRIP_HEIGHT;
}

void p_waitForStart() {
	char* score = "LAST SCORE:       ";
	// pointer to where the score value will be itoa'd
	char* scoreValue = score + 12;
	
	screen_init();
	
	itoa( scoreValue, p_lastScore );
	slPrint( score, slLocate( 11, 2 ) );
	
	slPrint( "Press A to start, and to raise gates", slLocate( 2, 13 ) );
	slPrint( "Saturnade by Sebastian Mihai, 2015", slLocate( 3, 27 ) );
	
	while( 1 ) {
		input_read();
		if( input_isTapped( BUTTON_A ) ) {
			return;
		}
	}
}

void p_playGame() {
	int frameCounter = 0;
	char* score = "SCORE:       ";
	// pointer to where the score value will be itoa'd
	char* scoreValue = score + 7;
	p_lastScore = 0;
	
	screen_init();
	
	slPrint( "Saturnade by Sebastian Mihai, 2015", slLocate( 3, 27 ) );
	
	obstacles_init();
	
	screen_flip();

	while( 1 ) {
		frameCounter++;
		input_read();
		if( input_isTapped( BUTTON_A ) ) {
			obstacles_shiftUp();
		}

		obstacles_update();

		screen_render( img_background, &p_playArea );
		obstacles_render();
		if( frameCounter % 8 < 4 ) {
			screen_render( img_ship, &p_ship );
		} else {
			screen_render( img_ship2, &p_ship );
		}

		p_lastScore = obstacles_getTotalCleared();
		itoa( scoreValue, p_lastScore );
		slPrint( score, slLocate( 16, 2 ) );

		if( obstacles_isCollisionWith( &p_ship ) ) {
			int i;
			for( i = 0; i < 60; i++ ) {
				slSynch();
			}
			
			// game over
			return;
		}

		screen_flip();
	}
}

// entry point
void ss_main() {
	p_init();
	sound_init();
	screen_init();
	input_init();
	obstacles_init();

	while( 1 ) {
		p_waitForStart();
		p_playGame();
	}
}

